/*
 * Crash Course STM32C0
 * Elektronika Praktyczna
 *
 * Przyklad 3. Konfiguracja RCC
 * plik main.c
 *
 */

#include "stm32c031xx.h"

void delayMilisec(uint32_t n);
void RCC_configClockSource(void);


int main()
{

	RCC->IOPENR |= RCC_IOPENR_GPIOAEN
				| RCC_IOPENR_GPIOBEN
				| RCC_IOPENR_GPIOCEN
				| RCC_IOPENR_GPIODEN;

	GPIOA->MODER &= ~GPIO_MODER_MODE5;
	GPIOA->MODER |= GPIO_MODER_MODE5_0;


	/* PA10 jako funkcja alternatywna AF3 (MCO2)*/
	GPIOA->MODER &= ~GPIO_MODER_MODE10;
	GPIOA->MODER |= GPIO_MODER_MODE10_1;
	GPIOA->AFR[1] |= GPIO_AFRH_AFSEL10_1 | GPIO_AFRH_AFSEL10_0;

	/* maksymalna szybkosc wyjscia PA10 */
	GPIOA->OSPEEDR |= GPIO_OSPEEDR_OSPEED10_1 | GPIO_OSPEEDR_OSPEED10_0;

	RCC_configClockSource();

	while(1)
	{

		GPIOA->BSRR = GPIO_BSRR_BS5;
		delayMilisec(100);
		GPIOA->BSRR = GPIO_BSRR_BR5;
		delayMilisec(100);

	}
}



void RCC_configClockSource(void)
{

	uint16_t i = 0;

	/* wlaczenie oscylatora HSE */
	RCC->CR |= RCC_CR_HSEON;

	while(i < 1000)
	{

		i++;

		/* test flagi gotowosci HSE */
		if((RCC->CR & RCC_CR_HSERDY) == RCC_CR_HSERDY)
			break;

	}

	if(i < 1000)
	{

		/* preskaler HCLK = 2 => HCLK = SYSCLK/2 = 24 MHz */
		RCC->CFGR |= RCC_CFGR_HPRE_3;

		/* preskaler PCLK = 2 => PCLK = HCLK/2 = 12 MHz */
		RCC->CFGR |= RCC_CFGR_PPRE_2;

		/* preskaler MCO2 = 64 => MCO2 = SYSCLK/64 = 750 kHz */
		RCC->CFGR |= RCC_CFGR_MCO2PRE_2 | RCC_CFGR_MCO2PRE_1;

		/* zrodlo sygnalu dla MCO2: SYSCLK */
		RCC->CFGR |= RCC_CFGR_MCO2SEL_0;

        /* zrodlo sygnalu dla SYSCLK: HSE */
        RCC->CFGR |= RCC_CFGR_SW_0;

        /* sprawdzenie czy SYSCLK przelaczyl sie na HSE */
        while(i < 10)
        {

        	/* test flagi gotowosci HSE */
            if((RCC->CFGR & RCC_CFGR_SWS_0) == RCC_CFGR_SWS_0)
                break;

            i++;

        }

        /* miejsce na obsluge bledu SWS */
        if(i == 10)
            while(1);

    } else {

        /* miejsce na obsluge bledu uruchomienia HSE */
        while(1);

    }

}

void delayMilisec(uint32_t n){

	for(uint32_t i = 0; i < 800*n; i++)
	{
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
		__NOP();
	}

}

