/*
 *  Crash Course STM32C0
 *  Elektronika Praktyczna
 *
 * 	Przyklad 4. Wysylka znakow przez UART2
 * 	plik hardware.c
 *
 */


#include "hardware.h"


void RCC_configClockSource(void)
{

	uint16_t i = 0;

	/* wlaczenie oscylatora HSE */
	RCC->CR |= RCC_CR_HSEON;

	while(i < 1000)
	{

		i++;

		/* test flagi gotowosci HSE */
		if((RCC->CR & RCC_CR_HSERDY) == RCC_CR_HSERDY)
			break;

	}

	if(i < 1000)
	{

		/* preskaler HCLK = 2 => HCLK = SYSCLK/2 = 24 MHz */
		RCC->CFGR |= RCC_CFGR_HPRE_3;

		/* preskaler PCLK = 2 => PCLK = HCLK/2 = 12 MHz */
		RCC->CFGR |= RCC_CFGR_PPRE_2;

        /* zrodlo sygnalu dla SYSCLK: HSE */
        RCC->CFGR |= RCC_CFGR_SW_0;

        /* sprawdzenie czy SYSCLK przelaczyl sie na HSE */
        while(i < 10)
        {

        /* test flagi gotowosci HSE */
        if((RCC->CFGR & RCC_CFGR_SWS_0) == RCC_CFGR_SWS_0)
           	break;

        i++;

    }

    /* miejsce na obsluge bledu SWS */
    if(i == 10)
		
        while(1);

    } else {

        /* miejsce na obsluge bledu uruchomienia HSE */
        while(1);

    }

}

void GPIO_config(void)
{

	/* wlaczenie taktowania blokow GPIO */
    RCC->IOPENR |= RCC_IOPENR_GPIOAEN
            | RCC_IOPENR_GPIOBEN
            | RCC_IOPENR_GPIOCEN
            | RCC_IOPENR_GPIODEN;

    /* ustawienie linii 5 jako wyjscie (LED) */
    GPIOA->MODER &= ~GPIO_MODER_MODE5;
    GPIOA->MODER |= GPIO_MODER_MODE5_0;

    /* ustawienie linii PA2 i PA3 jako AF1 (UART2) */
    GPIOA->MODER &= ~(GPIO_MODER_MODE2 | GPIO_MODER_MODE3);
    GPIOA->MODER |= GPIO_MODER_MODE2_1 | GPIO_MODER_MODE3_1;
    GPIOA->AFR[0] |= GPIO_AFRL_AFSEL2_0 | GPIO_AFRL_AFSEL3_0;

}

void UART_config(void)
{

    /* wlaczenie taktowania bloku USART2 */
    RCC->APBENR1 |= RCC_APBENR1_USART2EN;

    /* obliczenie i ustawienie preskalera dla 9600 bps, 8N1 */
    /* nadprobkowanie = 16 (domyslne), taktowanie z PCLK */
    /* PCLK = 12 MHz */
    USART2->BRR = 120000 / 96;

    /* wlaczenie nadajnika */
    USART2->CR1 = USART_CR1_TE;

    /* wlaczenie interfejsu */
    USART2->CR1 |= USART_CR1_UE;

}

void UART_print_chars(char *str)
{

	uint8_t timeout;

    /* sprawdzenie gotowosci bufora do wysylki */
    if ((USART2->ISR & USART_ISR_TXE_TXFNF) == USART_ISR_TXE_TXFNF){

    	/* wysylka znakow az do osiagniecia konca stringa */
    	/* warunek konca: zawartosc znaku rowna 0x00 */
        while (*str){

            /* zapis bajtu do rejestru nadawczego */
        	/* nastepnie: postinkrementacja wskaznika "w locie" */
            USART2->TDR = *(str++);

            /* timeout ~100 us * 12 = ~1,2 ms */
            timeout = 12;

            /*  oczekiwanie na zwolnienie bufora TDR */
            while (!(USART2->ISR & USART_ISR_TXE_TXFNF)){

            	if(timeout > 0){

            		delayMicrosec(100);
            		timeout--;

            	}else{

            		/* "brutalny" sposob obslugi bledu */
            		/* wlaczenie LED i wejscie w petle nieskonczona */
                    GPIOA->BSRR = GPIO_BSRR_BS5;
                	while(1);

            	}
            }

        }

        /* kolejny timeout ~1,2 ms */
		timeout = 12;

		/* oczekiwanie na ustawienie flagi zakonczenia transmisji */
		while (!(USART2->ISR & USART_ISR_TC)){

			if(timeout > 0){

				delayMicrosec(100);
				timeout--;

			}else{

				/* "brutalny" sposob obslugi bledu */
				GPIOA->BSRR = GPIO_BSRR_BS5;
				while(1);
			}
		}
    }
}

