/*
 *  Crash Course STM32C0
 *  Elektronika Praktyczna
 *
 *  Przyklad 7. Komunikacja przez SPI + NVIC + EXTI
 * 	plik hardware.c
 *
 */


#include "hardware.h"

void RCC_configClockSource(void)
{

	uint16_t i = 0;

	/* wlaczenie oscylatora HSE */
	RCC->CR |= RCC_CR_HSEON;

	while(i < 1000)
	{

		i++;

		/* test flagi gotowosci HSE */
		if((RCC->CR & RCC_CR_HSERDY) == RCC_CR_HSERDY)
			break;

	}

	if(i < 1000)
	{

		/* preskaler HCLK = 2 => HCLK = SYSCLK/2 = 24 MHz */
		RCC->CFGR |= RCC_CFGR_HPRE_3;

		/* preskaler PCLK = 2 => PCLK = HCLK/2 = 12 MHz */
		RCC->CFGR |= RCC_CFGR_PPRE_2;

        /* zrodlo sygnalu dla SYSCLK: HSE */
        RCC->CFGR |= RCC_CFGR_SW_0;

        /* sprawdzenie czy SYSCLK przelaczyl sie na HSE */
        while(i < 10)
        {

        /* test flagi gotowosci HSE */
        if((RCC->CFGR & RCC_CFGR_SWS_0) == RCC_CFGR_SWS_0)
           	break;

        i++;

    }

    /* miejsce na obsluge bledu SWS */
    if(i == 10)
		
        while(1);

    } else {

        /* miejsce na obsluge bledu uruchomienia HSE */
        while(1);

    }

}

void GPIO_config(void)
{

    RCC->IOPENR |= RCC_IOPENR_GPIOAEN
            | RCC_IOPENR_GPIOBEN
            | RCC_IOPENR_GPIOCEN
            | RCC_IOPENR_GPIODEN;

    /* ustawienie linii 5 jako wyjscie (LED) */
    GPIOA->MODER &= ~GPIO_MODER_MODE5;
    GPIOA->MODER |= GPIO_MODER_MODE5_0;

    /* ustawienie linii PA2 i PA3 jako AF1 (UART2) */
    GPIOA->MODER &= ~(GPIO_MODER_MODE2 | GPIO_MODER_MODE3);
    GPIOA->MODER |= GPIO_MODER_MODE2_1 | GPIO_MODER_MODE3_1;
    GPIOA->AFR[0] |= GPIO_AFRL_AFSEL2_0 | GPIO_AFRL_AFSEL3_0;

    /* ustawienie linii PA5...7 jako AF0 (SPI1) */
    GPIOA->MODER &= ~(GPIO_MODER_MODE7 | GPIO_MODER_MODE6 | GPIO_MODER_MODE5);
    GPIOA->MODER |= GPIO_MODER_MODE7_1 | GPIO_MODER_MODE6_1 | GPIO_MODER_MODE5_1;
    GPIOA->OSPEEDR |= GPIO_OSPEEDR_OSPEED7 | GPIO_OSPEEDR_OSPEED6 | GPIO_OSPEEDR_OSPEED5;
    GPIOA->AFR[0] &= ~(GPIO_AFRL_AFSEL5 | GPIO_AFRL_AFSEL6 | GPIO_AFRL_AFSEL7);

    /* linia PB0 jako wyjscie (CS) */
    GPIOB->BSRR = GPIO_BSRR_BS0;
    GPIOB->MODER &= ~GPIO_MODER_MODE0;
    GPIOB->MODER |= GPIO_MODER_MODE0_0;

    /* linia PA10 jako wejscie cyfrowe dla EXTI */
    GPIOA->MODER &= ~GPIO_MODER_MODE10;


}

void UART_config(void)
{

    /* wlaczenie taktowania bloku USART2 */
    RCC->APBENR1 |= RCC_APBENR1_USART2EN;

    /* obliczenie i ustawienie preskalera dla 9600 bps, 8N1 */
    /* nadprobkowanie = 16 (domyslne), taktowanie z PCLK */
    /* PCLK = 12 MHz */
    USART2->BRR = 120000 / 96;

    /* wlaczenie nadajnika i odbiornika */
    USART2->CR1 = USART_CR1_TE | USART_CR1_RE;

    /* wlaczenie interfejsu */
    USART2->CR1 |= USART_CR1_UE;

}

void UART_print_chars(char *str)
{

	uint8_t timeout;

    /* sprawdzenie czy bufor jest gotowy do wysylki */
    if ((USART2->ISR & USART_ISR_TXE_TXFNF) == USART_ISR_TXE_TXFNF){

    	/* wsylka znakow az do osiagniecia konca stringa */
    	/* warunek konca: zawartosc znaku rowna 0x00 */
        while (*str){

            /* zapis bajtu do rejestru nadawczego */
        	/* nastepnie: postinkrementacja wskaznika "w locie" */
            USART2->TDR = *(str++);

            /* timeout ~100 us * 12 = ~1,2 ms */
            timeout = 12;

            /*  oczekiwanie na zwolnienie bufora TDR */
            while (!(USART2->ISR & USART_ISR_TXE_TXFNF)){

            	if(timeout > 0){

            		delayMicrosec(100);
            		timeout--;

            	}else{

            		/* "brutalny" sposob obslugi bledu */
            		/* wlaczenie LED i wejscie w petle nieskonczona */
                    GPIOA->BSRR = GPIO_BSRR_BS5;
                	while(1);

            	}
            }

        }

        /* kolejny timeout ~1,2 ms */
		timeout = 12;

		/* oczekiwanie na ustawienie flagi zakonczenia transmisji */
		while (!(USART2->ISR & USART_ISR_TC)){

			if(timeout > 0){

				delayMicrosec(100);
				timeout--;

			}else{

				/* "brutalny" sposob obslugi bledu */
				GPIOA->BSRR = GPIO_BSRR_BS5;
				while(1);
			}
		}
    }
}

void UART_print_num(int16_t num)
{
	int16_t dig10000, dig1000, dig100, dig10, dig1;
	int16_t sgn = 0;

	char str[7];

	if(num < 0){
		num = -num;
		sgn = 1;
	}

	dig10000 = num / 10000;
	dig1000 = (num % 10000) / 1000;
	dig100 = (num % 1000) / 100;
	dig10 = (num % 100) / 10;
	dig1 = (num % 10);

	if(sgn)
		str[0] = '-';
	else
		str[0] = ' ';

	str[1] = '0' + dig10000;
	str[2] = '0' + dig1000;
	str[3] = '0' + dig100;
	str[4] = '0' + dig10;
	str[5] = '0' + dig1;
	str[6] = 0x00;

	UART_print_chars(str);
}



void SPI_config()
{

	/* wlaczenie taktowania SPI1 */
	RCC->APBENR2 |= RCC_APBENR2_SPI1EN;

	/* baudrate: ~1,5 MHz */
	SPI1->CR1 = SPI_CR1_BR_1;

	/* Tryb SPI: 3 (1,1) */
	SPI1->CR1 |= SPI_CR1_CPHA | SPI_CR1_CPOL;

	/* upewniamy sie, ze dane beda wysylane od MSB do LSB */
	SPI1->CR1 &= ~SPI_CR1_LSBFIRST;

	/* programowa obsluga linii CE (NSS) */
	SPI1->CR1 |= SPI_CR1_SSM | SPI_CR1_SSI;

	/* SPI w trybie master */
	SPI1->CR1 |= SPI_CR1_MSTR;

	/* transmisja slow 8-bitowych */
	SPI1->CR2 |= SPI_CR2_DS_2 | SPI_CR2_DS_1 | SPI_CR2_DS_0;

	/* ustawienie momentu wyzwalania zdarzenia RXNE (8 bitow) */
	SPI1->CR2 |= SPI_CR2_FRXTH;

	/* wlaczenie bloku SPI */
	SPI1->CR1 |= SPI_CR1_SPE;
}

void SPI_duplex_transaction(uint8_t *out, uint8_t *in, uint8_t len)
{

	/* CS = 0 */
	GPIOB->BSRR = GPIO_BSRR_BR0;

	uint16_t tmt = 500;

	for(uint8_t i = 0; i < len; i++)
	{

		/* oczekiwanie na pusty bufor nadawczy */
		while (!(SPI1->SR & SPI_SR_TXE))
		{};

		/* zapis bajtu do wysylki */
		*(uint8_t *)&(SPI1->DR) = out[i];

		/* oczekiwanie na zakonczenie odbioru */
		while(((SPI1->SR & SPI_SR_RXNE) != SPI_SR_RXNE) && (tmt > 0)){
			tmt--;
		}

		/* odczyt odebranego bajtu z bufora */
		in[i] = (uint8_t)(SPI1->DR);

	}

	/* CS = 1 */
	GPIOB->BSRR = GPIO_BSRR_BS0;

}


void NVIC_Config(void){

	/* wywolanie funkcji CMSIS - globalne zezwolenie przerwan */
	__enable_irq();
	
	/* ustawienie priorytetu przerwania EXTI4...15 */
	NVIC_SetPriority(EXTI4_15_IRQn, 1);
	
	/* "odmaskowanie" przerwania EXTI4...15 */
	NVIC_EnableIRQ(EXTI4_15_IRQn);

}

void EXTI_config(void){

	/* "odmaskowanie" przerwania od EXTI */
	EXTI->IMR1 |= EXTI_IMR1_IM10;

	/* przewanie generowane na zboczu narastajacym */
	EXTI->RTSR1 = EXTI_RTSR1_RT10;
	
	/* wyzerowanie bitow EXTICR => podpiecie linii PA10 */
	EXTI->EXTICR[2] &= ~EXTI_EXTICR3_EXTI10;

}

