/*
 * Crash Course STM32C0
 * Elektronika Praktyczna
 *
 * Przyklad 7. Komunikacja przez SPI + NVIC + EXTI
 * plik lis2dw12_driver.c
 *
 */

#include "lis2dw12_driver.h"

void LIS2DW12_startup(void){

	uint8_t out[2], in[2];

	/* odczyt (R/W=1) rejestru WHO_AM_I (adres=0x0F) */
	out[0] = 0x80 | 0x0F;
	out[1] = 0x00;
	SPI_duplex_transaction(out, in, 2);

	/* ustawienie (R/W=0) rejestru CTRL1 (High Performance Mode, 14 bit/25 Hz) */
	out[0] = 0x00 | 0x20;
	out[1] = 0x34;
	SPI_duplex_transaction(out, in, 2);

	/* ustawienie rejestru CTRL6 (Low Noise) */
	out[0] = 0x00 | 0x25;
	out[1] = 0x04;
	SPI_duplex_transaction(out, in, 2);

}

void LIS2DW12_read_XYZ(int16_t *x, int16_t *y, int16_t *z){

	/* odczyt rejestrow wyjsciowych akcelerometru */
	/* poczawszy od adresu 0x28 */
	uint8_t out[7], in[7];

	out[0] = 0x80 | 0x28;
	for(uint8_t i = 1; i < 7; i++) out[i] = 0x00;


	SPI_duplex_transaction(out, in, 7);

	*x = (int16_t)(in[2] << 8) | in[1];
	*y = (int16_t)(in[4] << 8) | in[3];
	*z = (int16_t)(in[6] << 8) | in[5];

}


uint8_t LIS2DW12_check_DRDY(void){

	uint8_t ret = 0;
	uint8_t out[2], in[2];

   	/* odczyt bitu DRDY(0x01) w rejestrze STATUS (0x27) */

	out[0] = 0x80 | 0x27;
   	out[1] = 0x00;

   	SPI_duplex_transaction(out, in, 2);

   	if((in[1] & 0x01)) ret = 1;

	return ret;
}


void LIS2DW12_enable_DRDY_INT1(void){

	uint8_t out[2], in[2];

   	/* zapis bitu INT1_DRDY(0x01) w rejestrze CTRL4_INT1_PAD_CTRL  (0x23) */

	out[0] = 0x00 | 0x23;
   	out[1] = 0x01;

   	SPI_duplex_transaction(out, in, 2);

   	/* zapis bitu INTERRUPTS_ENABLE(0x20) w rejestrze CTRL7(0x3F) */

	out[0] = 0x00 | 0x3F;
   	out[1] = 0xA0;

   	SPI_duplex_transaction(out, in, 2);

   	/* zerowanie bitu BDU(0x20) w rejestrze CTRL2(0x21) */

	out[0] = 0x00 | 0x21;
   	out[1] = 0x04;

   	SPI_duplex_transaction(out, in, 2);
}

