/*
 * Crash Course STM32C0
 * Elektronika Praktyczna
 *
 * Przyklad 8. Komunikacja przez I2C + NVIC + EXTI
 * plik hardware.c
 *
 */


#include "hardware.h"

void RCC_configClockSource(void)
{

	uint16_t i = 0;

	/* wlaczenie oscylatora HSE */
	RCC->CR |= RCC_CR_HSEON;

	while(i < 1000)
	{

		i++;

		/* test flagi gotowosci HSE */
		if((RCC->CR & RCC_CR_HSERDY) == RCC_CR_HSERDY)
			break;

	}

	if(i < 1000)
	{

		/* preskaler HCLK = 2 => HCLK = SYSCLK/2 = 24 MHz */
		RCC->CFGR |= RCC_CFGR_HPRE_3;

		/* preskaler PCLK = 2 => PCLK = HCLK/2 = 12 MHz */
		RCC->CFGR |= RCC_CFGR_PPRE_2;

        /* zrodlo sygnalu dla SYSCLK: HSE */
        RCC->CFGR |= RCC_CFGR_SW_0;

        /* sprawdzenie czy SYSCLK przelaczyl sie na HSE */
        while(i < 10)
        {

        /* test flagi gotowosci HSE */
        if((RCC->CFGR & RCC_CFGR_SWS_0) == RCC_CFGR_SWS_0)
           	break;

        i++;

    }

    /* miejsce na obsluge bledu SWS */
    if(i == 10)
		
        while(1);

    } else {

        /* miejsce na obsluge bledu uruchomienia HSE */
        while(1);

    }

}

void GPIO_config(void)
{

    RCC->IOPENR |= RCC_IOPENR_GPIOAEN
            | RCC_IOPENR_GPIOBEN
            | RCC_IOPENR_GPIOCEN
            | RCC_IOPENR_GPIODEN;

    /* ustawienie linii 5 jako wyjscie (LED) */
    GPIOA->MODER &= ~GPIO_MODER_MODE5;
    GPIOA->MODER |= GPIO_MODER_MODE5_0;

    /* ustawienie linii PA2 i PA3 jako AF1 (UART2) */
    GPIOA->MODER &= ~(GPIO_MODER_MODE2 | GPIO_MODER_MODE3);
    GPIOA->MODER |= GPIO_MODER_MODE2_1 | GPIO_MODER_MODE3_1;
    GPIOA->AFR[0] |= GPIO_AFRL_AFSEL2_0 | GPIO_AFRL_AFSEL3_0;

    /* linia PB0 jako wyjscie (CS) */
    GPIOB->BSRR = GPIO_BSRR_BS0;
    GPIOB->MODER &= ~GPIO_MODER_MODE0;
    GPIOB->MODER |= GPIO_MODER_MODE0_0;

    /* linia PA10 jako wejscie cyfrowe dla EXTI */
    GPIOA->MODER &= ~GPIO_MODER_MODE10;

    /* linie PB8...9 jako AF6 (I2C1) */
    GPIOB->MODER &= ~(GPIO_MODER_MODE8 | GPIO_MODER_MODE9);
    GPIOB->MODER |= GPIO_MODER_MODE8_1 | GPIO_MODER_MODE9_1;
    GPIOB->AFR[1] |= (0x06 << GPIO_AFRH_AFSEL9_Pos) | (0x06 << GPIO_AFRH_AFSEL8_Pos);


}

void UART_config(void)
{

    /* wlaczenie taktowania bloku USART2 */
    RCC->APBENR1 |= RCC_APBENR1_USART2EN;

    /* obliczenie i ustawienie preskalera dla 9600 bps, 8N1 */
    /* nadprobkowanie = 16 (domyslne), taktowanie z PCLK */
    /* PCLK = 12 MHz */
    USART2->BRR = 120000 / 96;

    /* wlaczenie nadajnika i odbiornika */
    USART2->CR1 = USART_CR1_TE | USART_CR1_RE;

    /* wlaczenie interfejsu */
    USART2->CR1 |= USART_CR1_UE;

}

void UART_print_chars(char *str)
{

	uint8_t timeout;

    /* sprawdzenie czy bufor jest gotowy do wysylki */
    if ((USART2->ISR & USART_ISR_TXE_TXFNF) == USART_ISR_TXE_TXFNF){

    	/* wsylka znakow az do osiagniecia konca stringa */
    	/* warunek konca: zawartosc znaku rowna 0x00 */
        while (*str){

            /* zapis bajtu do rejestru nadawczego */
        	/* nastepnie: postinkrementacja wskaznika "w locie" */
            USART2->TDR = *(str++);

            /* timeout ~100 us * 12 = ~1,2 ms */
            timeout = 12;

            /*  oczekiwanie na zwolnienie bufora TDR */
            while (!(USART2->ISR & USART_ISR_TXE_TXFNF)){

            	if(timeout > 0){

            		delayMicrosec(100);
            		timeout--;

            	}else{

            		/* "brutalny" sposob obslugi bledu */
            		/* wlaczenie LED i wejscie w petle nieskonczona */
                    GPIOA->BSRR = GPIO_BSRR_BS5;
                	while(1);

            	}
            }

        }

        /* kolejny timeout ~1,2 ms */
		timeout = 12;

		/* oczekiwanie na ustawienie flagi zakonczenia transmisji */
		while (!(USART2->ISR & USART_ISR_TC)){

			if(timeout > 0){

				delayMicrosec(100);
				timeout--;

			}else{

				/* "brutalny" sposob obslugi bledu */
				GPIOA->BSRR = GPIO_BSRR_BS5;
				while(1);
			}
		}
    }
}

void UART_print_num(int16_t num)
{
	int16_t dig10000, dig1000, dig100, dig10, dig1;
	int16_t sgn = 0;

	char str[7];

	if(num < 0){
		num = -num;
		sgn = 1;
	}

	dig10000 = num / 10000;
	dig1000 = (num % 10000) / 1000;
	dig100 = (num % 1000) / 100;
	dig10 = (num % 100) / 10;
	dig1 = (num % 10);

	if(sgn)
		str[0] = '-';
	else
		str[0] = ' ';

	str[1] = '0' + dig10000;
	str[2] = '0' + dig1000;
	str[3] = '0' + dig100;
	str[4] = '0' + dig10;
	str[5] = '0' + dig1;
	str[6] = 0x00;

	UART_print_chars(str);
}


void NVIC_Config(void){

	/* wywolanie funkcji CMSIS - globalne zezwolenie przerwan */
	__enable_irq();

	/* ustawienie priorytetu przerwania EXTI4...15 */
	NVIC_SetPriority(EXTI4_15_IRQn, 1);

	/* "odmaskowanie" przerwania EXTI4...15 */
	NVIC_EnableIRQ(EXTI4_15_IRQn);

}

void EXTI_config(void){

	/* "odmaskowanie" przerwania od EXTI */
	EXTI->IMR1 |= EXTI_IMR1_IM10;

	/* przewanie generowane na zboczu narastajacym */
	EXTI->RTSR1 = EXTI_RTSR1_RT10;

	/* wyzerowanie bitow EXTICR => podpiecie linii PA10 */
	EXTI->EXTICR[2] &= ~EXTI_EXTICR3_EXTI10;

}


void I2C_config(uint8_t addr){

	/* wybieramy SYSCLK = 48 MHz jako zrodlo sygnalu taktowania I2C1 */
	RCC->CCIPR |= RCC_CCIPR_I2C1SEL_0;

	/* wlaczenie taktowania interfejsu od strony szyny systemowej */
	RCC->APBENR1 |= RCC_APBENR1_I2C1EN;

	/* konfiguracja timingu dla f_SCLK = 100 kHz @ I2CCLK = 48 MHz */
	I2C1->TIMINGR = (uint32_t)(0x0B << I2C_TIMINGR_PRESC_Pos) |
					(uint32_t)(0x13 << I2C_TIMINGR_SCLL_Pos) |
					(uint32_t)(0x0F << I2C_TIMINGR_SCLH_Pos) |
					(uint32_t)(0x02 << I2C_TIMINGR_SDADEL_Pos) |
					(uint32_t)(0x04 << I2C_TIMINGR_SCLDEL_Pos);

	/* wlaczenie peryferium i zezwolenie na przerwanie od RX */
	I2C1->CR1 = I2C_CR1_PE | I2C_CR1_RXIE;

	/* wlaczenie opcji autmatycznego konczenia transmisji */
	/* ustawienie adresu slave'a */
	I2C1->CR2 = I2C_CR2_AUTOEND | (addr << I2C_CR2_SADD_Pos);

}


void I2C_write_bytes(uint8_t *ptr, uint8_t n){

	uint16_t tout = 0;

	/* sprawdzenie warunku rozpoczecia kolejnej transmisji */
	while((I2C1->CR2 & I2C_CR2_START) == I2C_CR2_START){

		if(tout < 1000){
			delayMicrosec(1);
		}else{
			break;
		}
		tout++;

	}

	/* ponowne sprawdzenie bitu START */
	if((I2C1->CR2 & I2C_CR2_START) != I2C_CR2_START){

		/* zerowanie bitu R/W (bedziemy nadawac dane) */
		I2C1->CR2 &= ~I2C_CR2_RD_WRN;

		/* ustawienie automatycznego zakonczenia transmisji */
		I2C1->CR2 |= I2C_CR2_AUTOEND;

		/* ustawienie dlugosci transmisji */
		I2C1->CR2 &= ~I2C_CR2_NBYTES;
		I2C1->CR2 |= (n << 16);

		/* sprawdzenie czy bufor nadawczy jest pusty */
		if((I2C1->ISR & I2C_ISR_TXE) == I2C_ISR_TXE){

			/* zapis pierwszego bajtu do rejestru wyjsciowego */
			I2C1->TXDR = ptr[0];

			/* nadanie warunku startu */
			I2C1->CR2 |= I2C_CR2_START;

		}

		/* wysylka pozostalych bajtow */
		for(uint8_t i = 1; i < n; i++){

			tout = 0;

			/* oczekiwanie na pusty bufor nadawczy */
			while((I2C1->ISR & I2C_ISR_TXE) != I2C_ISR_TXE){

				if(tout < 1000){

					delayMicrosec(1);

				}else{

					break;
				}

				tout++;

			}

			/* jezeli bufor pusty */
			if((I2C1->ISR & I2C_ISR_TXE) == I2C_ISR_TXE){

				/* nadawanie kolejnych bajtow */
				I2C1->TXDR = ptr[i];

				tout = 0;

				/* oczekiwanie na pusty bufor nadawczy */
				while(!(I2C1->ISR & I2C_ISR_TXE)){

					if(tout < 1000){

						delayMicrosec(1);

					}else{

						break;
					}

					tout++;
				}

			}else{
				/* miejsce na obsluge bledu zajetosci bufora */
			}
		}
	}else{
		/* miejsce na obsluge bledu bitu START */
	}
}

void I2C_read_bytes(uint8_t reg, uint8_t *ptr, uint8_t n){

	uint16_t tout = 0;

	/* sprawdzenie warunku rozpoczecia kolejnej transmisji */
	while((I2C1->CR2 & I2C_CR2_START) == I2C_CR2_START){

		if(tout < 1000){

			delayMicrosec(1);

		}else{

			break;

		}
		tout++;

	}

	/* ponowne sprawdzenie bitu START */
	if((I2C1->CR2 & I2C_CR2_START) != I2C_CR2_START){

		I2C1->CR2 &= ~I2C_CR2_RD_WRN;

		I2C1->CR2 &= ~I2C_CR2_AUTOEND;
		I2C1->CR2 &= ~I2C_CR2_NBYTES;
		I2C1->CR2 |= (1 << 16);

		if((I2C1->ISR & I2C_ISR_TXE) == I2C_ISR_TXE){

			I2C1->TXDR = reg;
			I2C1->CR2 |= I2C_CR2_START;
		}

		tout = 0;

		while((I2C1->CR2 & I2C_CR2_START) == I2C_CR2_START){

			if(tout < 1000){

				delayMicrosec(1);

			}else{

				break;

			}

			tout++;

		}

		if((I2C1->CR2 & I2C_CR2_START) != I2C_CR2_START){

			I2C1->CR2 |= I2C_CR2_RD_WRN | I2C_CR2_AUTOEND;
			I2C1->CR2 &= ~I2C_CR2_NBYTES;
			I2C1->CR2 |= (n << 16);

			/* powtorzony start */
			I2C1->CR2 |= I2C_CR2_START;

			for(uint8_t i = 0; i < n; i++){

				tout = 0;

				/* oczekiwanie na dane w buforze odbiorczym */
				while((I2C1->ISR & I2C_ISR_RXNE) != I2C_ISR_RXNE){

					if(tout < 1000){
						delayMicrosec(1);
					}else{
						break;
					}

					tout++;

				}

				/* ponowne sprawdzenie bufora odbiorczego */
				if((I2C1->ISR & I2C_ISR_RXNE) == I2C_ISR_RXNE){

					/* odczyt bajtu z bufora */
					ptr[i] = I2C1->RXDR;

				}else{
					/* przykladowa obsluga przypadku braku danych w buforze */
					ptr[i] = 0;

				}
			}
		}else{
			/* miejsce na obsluge bledu zajetosci bufora */
		}
	}else{
		/* miejsce na obsluge bledu zajetosci bufora */
	}


}


